

set -ex



python -V
pydoc -h
python-config --help
export PATH=${CONDA_PREFIX}/${HOST}/sysroot/bin:${CONDA_PREFIX}/${HOST}/sysroot/usr/bin:${PATH}
export LD_LIBRARY_PATH=${CONDA_PREFIX}/lib:${CONDA_PREFIX}/${HOST}/sysroot/lib64:${CONDA_PREFIX}/${HOST}/sysroot/usr/lib:/usr/lib64:/usr/lib
export DISPLAY=localhost:1
python tests.py
idle -h
python -c "import sysconfig; print sysconfig.get_config_var('CC')"
python -c 'import ssl; print(ssl.get_default_verify_paths())' | grep ${CONDA_PREFIX}
exit 0
